      program demo_PDSYEV
!     ==================
!  +-------------------------------------------------------------------+
!  |  Demo program for using the ScaLAPACK driver routine PDSYEV       |
!  |  for calculating eigenvalues and eigenvectors                     |
!  |  of a real symmetric matrix.                                      |
!  |  The following parameters are used:                               |
!  |  m_A = n, n_A = n, mb_A = nb, nb_A = nb, i_A = 1, j_A = 1         |
!  |  m_Z = n, n_Z = n, mb_Z = nb, nb_Z = nb, i_Z = 1, j_Z = 1         |
!  |  csrc_A = 0, rsrc_A= 0,  csrc_B = 0, rsrc_B= 0                    |
!  |  The matrix A  is chosen as                                       |
!  |  A(i,j) = 1./(mati + matj*abs(i-j)                                |
!  +-------------------------------------------------------------------+

      implicit none 

!  parameter constants
      integer           max_al, max_n, max_w
      parameter        (max_al=1100000, max_n=2000,  max_w=1100000)
!  variables for definition of matrix A 
      real*8 mati,matj
      common/matdat/mati,matj

!  local variables
      real*8    al(max_al), al_s(max_al), zl(max_al), 
     :          rl(max_al), w(max_n), work(max_w), err
      integer   ctxt, ctxt_sys, ctxt_all,
     :          nproc, nprow, npcol, myid, myrow, mycol,
     :          n, nb, lel1, lel2, 
     :          m_al, n_al, llda, desc_A(9), 
     :          ipr, ipc, il, jl, i, j, k, info, lwork, 
     :          NUMROC, INDXL2G, INDXG2L, INDXG2P

!  calling BLACS_PINFO(myid,nproc) initializes MPI and returns size and rank
      call BLACS_PINFO(myid,nproc)
      call BLACS_GET( 0, 0, ctxt_sys )
      ctxt_all = ctxt_sys
      call BLACS_GRIDINIT( ctxt_all, 'C', nproc, 1)
! input parameters
      n = 1000; nb = 24 
! Parameters for processor grid
      nprow=2; npcol=2
! Set up a process grid of size nprow*npcol
      if (nprow*npcol.gt.nproc) then
         write(6,*) 'nproc = ',nproc,' less then nprow*npcol = '
      end if
      ctxt = ctxt_sys; call BLACS_GRIDINIT( ctxt, 'C', nprow, npcol)
! Processes not belonging to the grid jump to the end of program
      if (ctxt.lt.0) go to 1000
! Get the process coordinates in the grid
      call BLACS_GRIDINFO( ctxt, nprow, npcol, myrow, mycol )
! number of rows and columns of local parts al, zl for A, Z
      m_al = NUMROC( n, nb, myrow, 0, nprow )
      n_al = NUMROC( n, nb, mycol, 0, npcol )
! size of workarray
      lel1 = nb**2*(n/(nb*nprow)+n/(nb*npcol)+3)
      lel2 = nb*((n-1)/(nb*nprow*npcol)+1)
      lwork = n*5 + max(2*n,lel1) + n*lel2 + 1

! Test for sufficient memory
      if (m_al*n_al.gt.max_al.or.
     :    n.gt.max_n.or.lwork.gt.max_w) then
       write(6,*)'not enough memory:  al ',m_al*n_al,max_al,
     : ' n',n,max_n,' lwork',lwork,max_w 
       end if
       
! initializing descriptors for the distributed matrices A, B:
      llda = max(1,m_al)
      call DESCINIT( desc_A, n, n, nb, nb,
     :               0, 0, ctxt, llda, info )

! initialize in parallel the local parts of A in al and in al_s
      do jl = 1 , n_al
         j = INDXL2G(jl,nb,mycol,0,npcol)
         do il = 1 , m_al
            i = INDXL2G(il,nb,myrow,0,nprow)
            al(il+m_al*(jl-1))= 1./(1.+5.*abs(i-j))
            al_s(il+m_al*(jl-1))= al(il+m_al*(jl-1))
         end do
      end do

! calculate eigenvalues and eigenvectors 
! setting lw = -1, no calculation is performed, only the needed
! workspace is returned in work(1)
      call PDSYEV( 'V', 'U', n, al, 1, 1, desc_A, w, 
     :                          zl, 1, 1, desc_A,
     :                          work, lwork, info )

! check the result
! matrix vector multiplication of coefficient matrix in al_s
! times eigenvector k, result is stored in global array R with 
! descriptor desc_A
      err = 0.d0
      do k = 1 , n
         call PDGEMV( 'N',n,n,  1.d0,al_s,1,1,desc_A,
     :                               zl,1,k,desc_A,1,
     :                          0.d0,rl,1,k,desc_A,1, )
         do i = 1 , n
            jl = INDXG2L(k,nb,0,0,npcol)
            ipc = INDXG2P(k,nb,0,0,npcol)
            il = INDXG2L(i,nb,0,0,nprow)
            ipr = INDXG2P(i,nb,0,0,nprow)
            if (ipr.eq.myrow.and.ipc.eq.mycol) then
               err = err +
     : abs(rl(il+llda*(jl-1)) - w(k)*zl(il+llda*(jl-1)))
            end if
         end do
      end do
      call dgsum2d( ctxt, 'A', ' ', 1, 1, err, 1, 0, 0 )
      if (myid.eq.0) then
            write(6,*)'Solving eigenproblem with PDSYEV with'
            write(6,*)'n =',n,' nb =',nb,' nprow,npcol =',nprow,npcol
            write(6,*)'error in eigensolution is',err/real(n**2)
      end if
 1000 continue
      call BLACS_EXIT(0)

      stop
      end

